@@ -9,8 +9,9 @@
 #include "../qrencode.h"
 #include "../qrinput.h"
 #include "../bitstream.h"
+#include "../qrencode_inner.h"
 
-#define testStart(__arg__) (testStartReal(__FUNCTION__, __arg__))
+#define testStart(__arg__) (testStartReal(__func__, __arg__))
 #define testEndExp(__arg__) (testEnd(!(__arg__)))
 
 static int tests = 0;
@@ -23,6 +24,23 @@ char levelChar[4] = {'L', 'M', 'Q', 'H'};
 const char *modeStr[5] = {"nm", "an", "8", "kj", "st"};
 
 void printQRinput(QRinput *input)
+{
+	QRinput_List *list;
+	unsigned char *p;
+	int i;
+
+	list = input->head;
+	while(list != NULL) {
+		p = list->data;
+		for(i=0; i<list->size; i++) {
+			printf("0x%02x,", list->data[i]);
+		}
+		list = list->next;
+	}
+	printf("\n");
+}
+
+void printQRinputInfo(QRinput *input)
 {
 	QRinput_List *list;
 	BitStream *b;
@@ -39,8 +57,10 @@ void printQRinput(QRinput *input)
 	}
 	printf("  chunks: %d\n", i);
 	b = QRinput_mergeBitStream(input);
-	printf("  bitstream-size: %d\n", BitStream_size(b));
-	BitStream_free(b);
+	if(b != NULL) {
+		printf("  bitstream-size: %d\n", BitStream_size(b));
+		BitStream_free(b);
+	}
 
 	list = input->head;
 	i = 0;
@@ -72,15 +92,16 @@ void testEnd(int result)
 	}
 }
 
-#define assert_exp(__exp__, __msg__...) \
-(void)({assertionNum++;if(!(__exp__)) {assertionFailed++; printf(__msg__);}})
+#define assert_exp(__exp__, ...) \
+{assertionNum++;if(!(__exp__)) {assertionFailed++; printf(__VA_ARGS__);}}
 
-#define assert_zero(__exp__, __msg__...) assert_exp((__exp__) == 0, __msg__)
-#define assert_nonzero(__exp__, __msg__...) assert_exp((__exp__) != 0, __msg__)
-#define assert_null(__ptr__, __msg__...) assert_exp((__ptr__) == NULL, __msg__)
-#define assert_nonnull(__ptr__, __msg__...) assert_exp((__ptr__) != NULL, __msg__)
-#define assert_equal(__e1__, __e2__, __msg__...) assert_exp((__e1__) == (__e2__), __msg__)
-#define assert_notequal(__e1__, __e2__, __msg__...) assert_exp((__e1__) != (__e2__), __msg__)
+#define assert_zero(__exp__, ...) assert_exp((__exp__) == 0, __VA_ARGS__)
+#define assert_nonzero(__exp__, ...) assert_exp((__exp__) != 0, __VA_ARGS__)
+#define assert_null(__ptr__, ...) assert_exp((__ptr__) == NULL, __VA_ARGS__)
+#define assert_nonnull(__ptr__, ...) assert_exp((__ptr__) != NULL, __VA_ARGS__)
+#define assert_equal(__e1__, __e2__, ...) assert_exp((__e1__) == (__e2__), __VA_ARGS__)
+#define assert_notequal(__e1__, __e2__, ...) assert_exp((__e1__) != (__e2__), __VA_ARGS__)
+#define assert_nothing(__exp__, ...) {printf(__VA_ARGS__); __exp__;}
 
 void testFinish(void)
 {
@@ -99,6 +120,31 @@ void report()
 	if(failed) exit(-1);
 }
 
+int ncmpBin(char *correct, BitStream *bstream, int len)
+{
+	int i, bit;
+
+	if(len != BitStream_size(bstream)) {
+		printf("Length is not match: %d, %d expected.\n", BitStream_size(bstream), len);
+		return -1;
+	}
+
+	for(i=0; i<len; i++) {
+		bit = (correct[i] == '1')?1:0;
+		if(bstream->data[i] != bit) return -1;
+	}
+
+	return 0;
+}
+
+int cmpBin(char *correct, BitStream *bstream)
+{
+	int len;
+
+	len = strlen(correct);
+	return ncmpBin(correct, bstream, len);
+}
+
 char *sprintfBin(int size, unsigned char *data)
 {
 	int i, j;
